// C Source File
// Created 15/02/2002; 14:56:56

#define USE_TI89              // Produce .89z File
#define USE_TI92PLUS          // Produce .9xz File

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files


#define COLONNE_DEF 10
#define LIGNE_DEF 10
#define BOMBE_DEF 10
#define VIE_DEF 1
#define MAX_VIE 10

/*
//=======================================================================================================================
//====================================================== Commentaires ===================================================
//=======================================================================================================================

Dans le terrain de mine:
	0  9: pas de bombe: on indique le nb de bombe autour
	10  19: y'a une bombe
Drapeau: on ajoute 20:
	20  29: drapeau sans bombe
 	30  39: drapeau avec bombe
 	50: case dcouverte: on y touche plus


*/


//=======================================================================================================================
//===================================================== Variables globales ===============================================
//=======================================================================================================================

	unsigned char case_light[]={0x3E, 0x3E, 0x3E, 0x3E, 0x3E, 0x00};  //light
	unsigned char case_dark[]={0x01, 0x01, 0x01, 0x01, 0x01, 0x3F};  //dark
	unsigned char bombe_light[]={0x3E, 0x36, 0x22, 0x36, 0x3E};  //light
	unsigned char bombe_dark[]={0x2B, 0x1D, 0x3F, 0x1D, 0x2B, 0x3F};  //dark
	unsigned char drapeau[]={0x08, 0x18, 0x38, 0x08, 0x1C};  //dark

//mise en variables globale pour mmoriser les valeurs du dernier lancement
	int nbcolonne=COLONNE_DEF;
	int nbligne=LIGNE_DEF;
	int nbvie=VIE_DEF;
	int nbbombe=BOMBE_DEF;
	int * * terrain=NULL;  //terrain de mine dont on ne connait pas encore les dimensions
	void *save_light;  //pour sauvegarder les crans light et dark
	void *save_dark;


//=======================================================================================================================
//===================================================== Dessine le terrain ==============================================
//=======================================================================================================================

void init(int posx, int posy) {

	register int j,i,k;
	int col, lig;

	unsigned char *case_terrain;

//initialisation du terrain de mine
	for (j=0;j<=nbligne+1;j++) {  //bordures
		terrain[0][j]=50;
		terrain[nbcolonne+1][j]=50;
	}
	for (i=1;i<=nbcolonne;i++) {
		terrain[i][0]=50;  //bordure
		for (j=1;j<=nbligne;j++)
			terrain[i][j]=0;  //terrain
		terrain[i][nbligne+1]=50;  //bordure
	}

//pose des bombes
	randomize();  //permet d'avoir un nombre vraiment alatoire

	for (i=0;i<nbbombe;i++) {
		while ( terrain[col=random(nbcolonne)+1][lig=random(nbligne)+1]>9 )  //tant qu'on tombera sur une bombe
			;
		terrain[col][lig]+=10;  //on ajoute une bombe
		for (j=-1;j<2;j++) {
			for (k=-1;k<2;k++)
				terrain[col+j][lig+k]++;  //on incrmente tout autour
		}
	}

//trac du terrain
	case_terrain=case_light;
	for (k=0;k<2;k++) {  //plan light puis plan dark
		SetPlane(k);
		clrscr();
		if (k==1)
			case_terrain=case_dark;
		for (j=1;j<=nbligne;j++) {
			for (i=1;i<=nbcolonne;i++)
				Sprite8(posx+8*i, posy+8*j+2, 6, case_terrain, GetPlane(k), SPRT_OR);
			DrawLine(posx+8,posy+8*j,LCD_WIDTH-posx-8,posy+8*j,A_NORMAL);
		}
		DrawLine(posx+8,LCD_HEIGHT-posy-8,LCD_WIDTH-posx-8,LCD_HEIGHT-posy-8,A_NORMAL);
		for (i=1;i<=nbcolonne+1;i++)
			DrawLine(posx+8*i,posy+8,posx+8*i,LCD_HEIGHT-posy-8,A_NORMAL);
	}
}

//=======================================================================================================================
//===================================================== Affiche une case =============================================
//=======================================================================================================================

void aff_case(int x, int y, int val) {
	SetPlane(LIGHT_PLANE);
	Sprite8(x-2, y, 6, case_light, GetPlane(LIGHT_PLANE), SPRT_XOR);
	DrawChar(x, y, '0'+val, A_NORMAL);
	SetPlane(DARK_PLANE);
	Sprite8(x-2, y, 6, case_dark, GetPlane(DARK_PLANE), SPRT_XOR);
	DrawChar(x, y, '0'+val, A_NORMAL);
}
//=======================================================================================================================
//===================================================== Recherche des zeros =============================================
//=======================================================================================================================

void zeros(int posx, int posy, int curs_x, int curs_y, int *nbcase) {
	register int i, j;

	terrain[curs_x][curs_y]=50;
	(*nbcase)--;
	
//affichage du zro
	aff_case(posx+8*curs_x+2,posy+8*curs_y+2,0);

//recherche d'autres zro tout autour
	for (i=-1;i<2;i++) {
		for (j=-1;j<2;j++) {
			if (terrain[curs_x+i][curs_y+j]>10)  //y'a un drapeau ou case dja dcouverte (il ne peut pas y avoir de bombe!)
				continue;
			if (terrain[curs_x+i][curs_y+j]==0)  //y'a un zro
				zeros(posx, posy, curs_x+i, curs_y+j, nbcase);  //tude de ce zro
			else {  //c'est un nombre
				aff_case(posx+8*(curs_x+i)+2,posy+8*(curs_y+j)+2,terrain[curs_x+i][curs_y+j]);
				terrain[curs_x+i][curs_y+j]=50;
				(*nbcase)--;
			}
		}  //au suivant
	}
	
	
}


//=======================================================================================================================
//================= Affiche un texte avec/sans ombre proportionnelement dans un cadre blanc ==================
//=======================================================================================================================

void aff_txt2(int posx, float posy, char texte[], int ombre) {
	int i,h,l;
	
	l=(FontGetSys()+2)*strlen(texte);//longueur du texte
	if (posx==-1)
		posx=LCD_WIDTH/2-l-ombre;  //on veut centrer le texte
	posx= ((posx+2*l+ombre>LCD_WIDTH) ? LCD_WIDTH-2*l-ombre : posx);  //en cas de dpassement (jamais de texte trop long)
	posy=LCD_HEIGHT*posy-(h=FontGetSys()+3)-ombre;  //j'utilise jamais de petits caractres : (2*hautfont)/2
																				//posy se fait par la proportionnel  l'cran 0<posy<1
																				//si y'a de l'ombre, on remonte de 1
	posy= ( (posy<0) ? 0 : ( (posy+2*h+ombre>LCD_HEIGHT) ? LCD_HEIGHT-2*h-ombre : posy) );  //en cas de dpassement

	SetPlane(DARK_PLANE);
	DrawStr(posx,posy,texte,A_REPLACE);
	SetPlane(LIGHT_PLANE);
	DrawStr(posx,posy,texte,A_REPLACE);

	for (i=1;i<3;i++) {  //cration du cadre
		if (posy-i>=0)
			DrawLine(( ( posx-2<0) ? 0 : posx-2 ),posy-i,( ( posx+2*l+1>LCD_WIDTH) ? LCD_WIDTH : posx+2*l+1),posy-i,A_REVERSE);  //effacement au dessus
		if (posy+2*h+i-1<=LCD_HEIGHT)
			DrawLine(( ( posx-2<0) ? 0 : posx-2 ),posy+2*h+i-1,( ( posx+2*l+1>LCD_WIDTH) ? LCD_WIDTH : posx+2*l+1),posy+2*h+i-1,A_REVERSE);  //effacement en dessous
		if (posx-i>=0)
			DrawLine(posx-i,posy,posx-i,posy+2*h-1,A_REVERSE);  //effacement  gauche
		if (posx+2*l+i-1<=LCD_WIDTH)
			DrawLine(posx+2*l+i-1,posy,posx+2*l+i-1,posy+2*h-1,A_REVERSE);  //effacement  droite
	}	

	if (ombre)
		DrawStr(posx+1,posy+1,texte,A_NORMAL);  //cre l'ombre !!

}

//=======================================================================================================================
//=========================== Affiche un texte avec/sans ombre proportionnelement  l'cran =============================
//=======================================================================================================================

void aff_txt(int posx, float posy, char texte[], int ombre) {//idem que prcdent, mais sans cadre
	int h,l;
	
	l=(FontGetSys()+2)*strlen(texte);
	if (posx==-1)
		posx=LCD_WIDTH/2-l-ombre;
	posx= ( (posx<0) ? 0 : ( (posx+2*l+ombre>LCD_WIDTH) ? LCD_WIDTH-2*l-ombre : posx) );

	posy=LCD_HEIGHT*posy-(h=FontGetSys()+3)-ombre;

	posy= ( (posy<0) ? 0 : ( (posy+2*h+ombre>LCD_HEIGHT) ? LCD_HEIGHT-2*h-ombre : posy) );

	SetPlane(DARK_PLANE);
	DrawStr(posx,posy,texte,A_NORMAL);
	SetPlane(LIGHT_PLANE);
	DrawStr(posx,posy,texte,A_NORMAL);

	if (ombre)
		DrawStr(posx+1,posy+1,texte,A_NORMAL);

}

//=======================================================================================================================
//================================================ Choix de la taille du terrain ========================================
//=======================================================================================================================


int choix_terrain(void) {
	
	int j;
	unsigned int key;
	int list[4]={nbcolonne, nbligne, nbbombe, nbvie};  //reprise des anciennes valeurs
	int list_max[4]={LCD_WIDTH/8,LCD_HEIGHT/8,0,MAX_VIE};
	int menu;
	char tmptxt[50];
	
	for (j=0;j<2;j++) {  //on efface tout
		SetPlane(j);
		clrscr();
	}
	aff_txt(-1,0.1,"Hib-Dmine",1);	
	menu=1;
	FontSetSys(F_6x8);
	list_max[2]=list_max[0]*list_max[1];


menu:
	for (j=0;j<2;j++) {  //affichage en noir: sur light et dark
		SetPlane(j);
		DrawStr(3,30,"Taille :", A_NORMAL);
		sprintf(tmptxt,"- nombre de colonnes : %2d",list[0]);
		DrawStr(6,40,tmptxt,( (menu==1) ? A_REVERSE : A_REPLACE ) );
		sprintf(tmptxt,"- nombre de lignes : %2d",list[1]);
		DrawStr(6,50,tmptxt,( (menu==2) ? A_REVERSE : A_REPLACE ) );
		sprintf(tmptxt,"Nombre de bombes : %2d",list[2]);
		DrawStr(3,60,tmptxt,( (menu==3) ? A_REVERSE : A_REPLACE ) );
		sprintf(tmptxt,"Nombre de vies : %2d",list[3]);
		DrawStr(3,70,tmptxt,( (menu==4) ? A_REVERSE : A_REPLACE ) );
	}
	key=ngetchx();
	if (key==KEY_UP)
		menu--;
	else if (key==KEY_DOWN)
		menu++;
	else if (key==KEY_ESC)
		return 0;
	else if (key==KEY_ENTER)
		goto suite;
	else if (key==KEY_RIGHT || key==KEY_LEFT) {  //on change une valeur
		list[menu-1]+=( (key==KEY_RIGHT) ? 1 : -1 );
		list[menu-1]=( (list[menu-1]>list_max[menu-1]) ? 1 : ( (list[menu-1]<1) ? list_max[menu-1] : list[menu-1] ) );  //dpassement: on retourne  l'oppos
		if (menu<3) {  //si on change de taille, on change le nombre de bombe max
			list_max[2]=list[0]*list[1];
			if (list[2]>list_max[2])
				list[2]=list_max[2];
		}
	}
	menu=( (menu>4) ? 1 : ( (menu<1) ? 4 : menu ) );  //dpassement: on retourne  l'oppos
	
	goto menu;

suite:  //fin: on enregistre les valeurs
	nbcolonne=list[0];
	nbligne=list[1];
	nbbombe=list[2];
	nbvie=list[3];
	
	return 1;
}

//=======================================================================================================================
//============================================= Allocation du terrain de mines ==========================================
//=======================================================================================================================

int alloc_terrain(void) {

	register int j,i;

	if ((terrain=calloc(nbcolonne+2,sizeof(int *)))==NULL)
		goto mem;  //si y'a un pb
	for (j=0;j<nbcolonne+2;j++) {
		if ((terrain[j]=calloc(nbligne+2,sizeof(int)))==NULL) {
			goto mem;
		}
	}
	return 1;  //pas de pb

mem:  //y'a eu une couille ...
	for (i=0;i<j;i++)  //on libre les terrains allou
		free(terrain[i]);
	free(terrain);

	for (j=0;j<2;j++) {  //on efface
		SetPlane(j);
		clrscr();
	}
	FontSetSys(F_8x10);
	aff_txt(-1,0.3,"Erreur",1);
	FontSetSys(F_6x8);
	aff_txt(-1,0.5,"L'allocation mmoire a",0);
	aff_txt(-1,0.6,"chou, srement due  un",0);
	aff_txt(-1,0.7,"manque de mmoire RAM.",0);
	ngetchx();
	return 0;
}


//=======================================================================================================================
//========================================== Allocation des crans de sauvegardes =======================================
//=======================================================================================================================

int alloc_ecran(void) {

	register int j;

	if ((save_light=malloc(LCD_SIZE))==NULL)
		goto mem;
	if ((save_dark=malloc(LCD_SIZE))!=NULL)
		return 1;  //pas de pb
	free(save_light);
mem:  //y'a eu une couille ...
	for (j=0;j<2;j++) {  //on efface
		SetPlane(j);
		clrscr();
	}
	FontSetSys(F_8x10);
	aff_txt(-1,0.3,"Erreur",1);
	FontSetSys(F_6x8);
	aff_txt(-1,0.5,"L'allocation mmoire a",0);
	aff_txt(-1,0.6,"chou, srement due  un",0);
	aff_txt(-1,0.7,"manque de mmoire RAM.",0);
	ngetchx();
	return 0;
}

//=======================================================================================================================
//=========================================================== Affichage de l'aide =======================================
//=======================================================================================================================

void aide(void) {
	int j;
	char txttemp[30];

//sauvegarde des crans
	memcpy(save_light,GetPlane(LIGHT_PLANE),LCD_SIZE);
	memcpy(save_dark,GetPlane(DARK_PLANE),LCD_SIZE);
	for (j=0;j<2;j++) {  //on efface tout
		SetPlane(j);
		clrscr();
	}
	FontSetSys(F_8x10);
	aff_txt(-1,0.1,"Aide",1);
	FontSetSys(F_6x8);
	sprintf(txttemp,"%c%c%c%c : dplacement",21,22,23,24);  // les quatres flches
	aff_txt(5,0.3,txttemp,0);
	if (TI89)  //la ti89 n'est pas assez grande...
		sprintf(txttemp,"%c + %c%c%c%c : dplac%c rapide",127,21,22,23,24,153);  // diamant + les quatres flches  et transpose
	else
		sprintf(txttemp,"%c + %c%c%c%c : dplacement rapide",21,22,23,24);  // diamant + les quatres flches
	aff_txt(5,0.4,txttemp,0);
	aff_txt(5,0.5,"F1: dcouverte",0);
	aff_txt(5,0.6,"F2: drapeau",0);
	aff_txt(5,0.7,"ESC: quitter",0);
	aff_txt(5,0.8,"F5: aide",0);
	aff_txt(5,0.9,"APPS: On/Off",0);

	ngetchx();
//on restaure les crans
	memcpy(GetPlane(LIGHT_PLANE),save_light,LCD_SIZE);
	memcpy(GetPlane(DARK_PLANE),save_dark,LCD_SIZE);
	
}


//=======================================================================================================================
//=========================================================== Menu principal ============================================
//=======================================================================================================================

int menu(void) {
	
	int j;
	int menu;
	unsigned int key;
	char txttemp[15];


	menu=1;
	sprintf(txttemp,"A propos de%c",160);  //affiche les pointill
	
deb:
	for (j=0;j<2;j++) {  //on efface tout
		SetPlane(j);
		clrscr();
	}
	FontSetSys(F_8x10);
	aff_txt(-1,0.1,"Hib-Dmine",1);

menu:
	for (j=0;j<2;j++) {
		SetPlane(j);
		DrawStr(20,35,"Jouer",( (menu==1) ? A_REVERSE : A_REPLACE ) );
		DrawStr(20,45,"Aide",( (menu==2) ? A_REVERSE : A_REPLACE ) );
		DrawStr(20,55,txttemp,( (menu==3) ? A_REVERSE : A_REPLACE ) );
		DrawStr(20,65,"Quitter",( (menu==4) ? A_REVERSE : A_REPLACE ) );
	}
	key=ngetchx();
	if (key==KEY_UP)
		menu--;
	else if (key==KEY_DOWN)
		menu++;
	else if (key==KEY_ESC)
		return 0;
	else if (key==KEY_ENTER) {
		if (menu==1)
			return 1;  //on joue
		if (menu==2) {  //affiche l'aide
			aide();
			FontSetSys(F_8x10);
		} else if (menu==3) {//a propos de...
			for (j=0;j<2;j++) {  //on efface tout
				SetPlane(j);
				clrscr();
			}
			FontSetSys(F_8x10);
			aff_txt(-1,0.1,txttemp,1);
			FontSetSys(F_6x8);

			aff_txt(10,0.29,"Hib-Dmine v1.0",0);
			aff_txt(10,0.41,"Programmeur:",0);
			aff_txt(5,0.5,"hibou <hibou@bonbon.net>",0);
			aff_txt(10,0.62,"Merci :",0);
			aff_txt(5,0.71,"Les quipes TIGCC & TICT",0);
			aff_txt(5,0.8,"Toute la communaut TI-fr",0);
			aff_txt(300,1,"Mai 2002",0);

			ngetchx();
			goto deb;  //faut retourner au dbut pour tout rafficher
		} else if (menu==4)
			return 0;  //on quitte
	}
	menu=( (menu>4) ? 1 : ( (menu<1) ? 4 : menu ) );  //dpassement: on retourne  l'oppos
	
	goto menu;

}

//=======================================================================================================================
//======================================================= Main Function =================================================
//=======================================================================================================================

void _main(void) {

	int nbcase;  //nb de cases qui reste  dminer
	int posx, posy;  //position du coin suprieur droit de terrain de mine
	int curs_x, curs_y;  //position du curseur
	int *val_case;  //pointeur pour ne pas a avoir a recalculer  chaque fois le no de la case !!
	int quitter;
	int j;
	int vies;  //nb de vies
	unsigned int key;  //no de touche

	
	char txttemp[30];  //texte temporaire pour sprintf
	GrayOn();  //on passe en niveaux de gris
	if (alloc_ecran()==0)  //allocation des ecrans de sauvegarde
		goto quit;  //on a eu un pb
	
deb:
	if (menu()==0)  //menu principal
		goto quit;  //on veut quitter le jeux

	if (choix_terrain()==0)  //choix du nb de lignes, colonnes, bombes et vies
		goto deb;  //on veut revenir en arrire

	if (alloc_terrain()==0)  //allocation du terrain de mine
		goto quit;  //on a eu un pb

	nbcase=nbcolonne*nbligne-nbbombe;
	vies=nbvie;
	quitter=0;
	FontSetSys(F_4x6);

//centrage du terrain dans l'cran
	posx=(LCD_WIDTH-8*nbcolonne)/2-8;
	posy=(LCD_HEIGHT-8*nbligne)/2-8;

	init(posx, posy);  //dessin du terrain

//centrage du curseur
	curs_x=nbcolonne/2+1;
	curs_y=nbligne/2+1;

joue:
	do {
		for (j=0;j<2;j++) {  //affichage du curseur : inversement des couleurs sur les deux crans
			SetPlane(j);
			DrawLine(posx+8*curs_x, posy+8*curs_y, posx+8*curs_x+8, posy+8*curs_y, A_XOR);
			DrawLine(posx+8*curs_x+8, posy+8*curs_y, posx+8*curs_x+8, posy+8*curs_y+8, A_XOR);
			DrawLine(posx+8*curs_x+8, posy+8*curs_y+8, posx+8*curs_x, posy+8*curs_y+8, A_XOR);
			DrawLine(posx+8*curs_x, posy+8*curs_y+8, posx+8*curs_x, posy+8*curs_y, A_XOR);
		}
	key=ngetchx();
		for (j=0;j<2;j++) {  //effacement du curseur : inversement des couleurs sur les deux crans
			SetPlane(j);
			DrawLine(posx+8*curs_x, posy+8*curs_y, posx+8*curs_x+8, posy+8*curs_y, A_XOR);
			DrawLine(posx+8*curs_x+8, posy+8*curs_y, posx+8*curs_x+8, posy+8*curs_y+8, A_XOR);
			DrawLine(posx+8*curs_x+8, posy+8*curs_y+8, posx+8*curs_x, posy+8*curs_y+8, A_XOR);
			DrawLine(posx+8*curs_x, posy+8*curs_y+8, posx+8*curs_x, posy+8*curs_y, A_XOR);
		}

		val_case=&(terrain[curs_x][curs_y]);  //pour viter de recalculer  chaque fois....
		if (key==KEY_RIGHT)
			curs_x++;
		else if (key==KEY_LEFT)
			curs_x--;
		else if (key==KEY_UP)
			curs_y--;
		else if (key==KEY_DOWN)
			curs_y++;
/*
		else if (key==KEY_SHIFT+KEY_RIGHT)
			curs_x+=5;
		else if (key==KEY_SHIFT+KEY_LEFT)
			curs_x-=5;
		else if (key==KEY_SHIFT+KEY_UP)
			curs_y-=5;
		else if (key==KEY_SHIFT+KEY_DOWN)
			curs_y+=5;
*/
		else if (key==KEY_F1 && *val_case<20) { //on veut dminer
			if ((*val_case)>9) {  //y'a une bombe
				quitter=2;
				*val_case=50;
			} else { //on affiche le nombre
				if (*val_case==0) { //on recherche tous les zeros
					zeros(posx, posy, curs_x, curs_y, &nbcase);
				} else {  //affichage normal
					aff_case(posx+8*curs_x+2, posy+8*curs_y+2,*val_case);
					*val_case=50;  //case affiche
					nbcase--;  //le nb de case  dcouvrir diminue
				}
				if (nbcase==0)  //on a gagn
					quitter=3;
			}
		} else if (key==KEY_F2  && *val_case<50) {  //drapeau
			if (*val_case<20) //y'avais pas de drapeau
				*val_case+=20;
			else //y'en avais un
				*val_case-=20;
			Sprite8(posx+8*curs_x, posy+8*curs_y+2, 5, drapeau, GetPlane(DARK_PLANE), SPRT_XOR);  //on inverse la drapeau
		} else if (key==KEY_F5) {  //un peu d'aide
			aide();
			FontSetSys(F_4x6);
		} else if (key==KEY_ESC) {  //on veut quitter
			quitter=1;
		} else if (key==KEY_APPS) { //metre la TI hors tension
			off();
		}
		
//si on atteint les bords...
		if (key==KEY_DIAMOND+KEY_LEFT || curs_x>nbcolonne)
			curs_x=1;
		else if (key==KEY_DIAMOND+KEY_RIGHT || curs_x<1)
			curs_x=nbcolonne;
		
		if (key==KEY_DIAMOND+KEY_UP || curs_y>nbligne)
			curs_y=1;
		else if (key==KEY_DIAMOND+KEY_DOWN || curs_y<1)
			curs_y=nbligne;
	
	}	while (quitter==0);  //on boucle tant qu'on veut pas quitter
	
	if (quitter==1) {  //on veut quitter
		memcpy(save_light,GetPlane(LIGHT_PLANE),LCD_SIZE);  //sauvegarde des crans
		memcpy(save_dark,GetPlane(DARK_PLANE),LCD_SIZE);

		SetPlane(DARK_PLANE);  //effacement de l'cran dark seulement
		clrscr();
		FontSetSys(F_6x8);
		aff_txt2(-1,0.3,"Veux-tu vraiment",0);
		aff_txt2(-1,0.5,"quitter ?",0);
		aff_txt2(-1,0.7,"Oui=[ENTER]   Non=[ESC]",0);
touche:
		key=ngetchx();
		if (key==KEY_ESC) {  //on veut pas quitter
			memcpy(GetPlane(LIGHT_PLANE),save_light,LCD_SIZE);  //restauration des cran
			memcpy(GetPlane(DARK_PLANE),save_dark,LCD_SIZE);
			FontSetSys(F_4x6);
			quitter=0;
			goto joue;  //on retoure dans la boucle
		} else if (key!=KEY_ENTER)  //on veut quitter
			goto touche;
	} else if (quitter==2) {  //on est tomb sur une bombe
		Sprite8(posx+8*curs_x, posy+8*curs_y+2, 6, bombe_dark, GetPlane(DARK_PLANE), SPRT_XOR);  //dessin d'une bombe qui a explos
		Sprite8(posx+8*curs_x, posy+8*curs_y+2, 5, bombe_light, GetPlane(LIGHT_PLANE), SPRT_XOR);
		memcpy(save_light,GetPlane(LIGHT_PLANE),LCD_SIZE);  //sauvegarde des crans
		memcpy(save_dark,GetPlane(DARK_PLANE),LCD_SIZE);
		ngetchx();  //pause pour admirer la connerie faite
		FontSetSys(F_8x10);
		SetPlane(DARK_PLANE);
		clrscr();
		aff_txt2(-1,0.3,"Boouumm !!",1);
		sprintf(txttemp,"Nombre de vies: %d",--vies);
		aff_txt2(-1,0.5,txttemp,1);

		ngetchx();

		if (vies>0) {  //on peut encore continuer  dminer
			memcpy(GetPlane(LIGHT_PLANE),save_light,LCD_SIZE);
			memcpy(GetPlane(DARK_PLANE),save_dark,LCD_SIZE);
			quitter=0;
			FontSetSys(F_4x6);
			goto joue;
		}
		aff_txt2(-1,0.7,"Perdu !",1);
		ngetchx();
	} else if (quitter==3) {  //on a gagn
		ngetchx();  //pause pour admirer le terrain dmin
		SetPlane(DARK_PLANE);
		clrscr();
		FontSetSys(F_8x10);
		aff_txt2(-1,0.5,"Terrain dmin !",1);
		ngetchx();
	}

	for (j=0;j<nbcolonne+2;j++)  //on libre le terrain de mine
		free(terrain[j]);
	free(terrain);
	goto deb;
	
quit:
	free(save_light);  //on libre les ecrans de sauvegarde
	free(save_dark);
	FontSetSys(F_8x10);
	for (j=0;j<2;j++) {
		SetPlane(j);
		clrscr();
	}
	aff_txt(300,1,"by hibou",1);  //mot de la fin
	aff_txt(-1,0.25,"Salut !",1);
	aff_txt(-1,0.5,"Et  la prochaine !",1);
	ngetchx();
	GrayOff();

}
